"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRepository = void 0;
const knex_utils_1 = require("knex-utils");
const NonUniqueResultError_1 = require("./NonUniqueResultError");
const NoEntityExistsError_1 = require("./NoEntityExistsError");
const dbSupportHelper_1 = require("./dbSupportHelper");
class AbstractRepository {
    constructor(knex, config) {
        var _a, _b, _c;
        this.knex = knex;
        this.tableName = config.tableName;
        this.idColumn = config.idColumn;
        this.defaultOrderBy = config.defaultOrderBy;
        this.columnsToFetch = config.columnsToFetch;
        this.columnsToFetchList = (_a = config.columnsToFetch) !== null && _a !== void 0 ? _a : config.columnsToFetch;
        this.columnsToFetchDetails = (_b = config.columnsToFetch) !== null && _b !== void 0 ? _b : config.columnsToFetch;
        this.columnsForCreate = (_c = config.columnsForCreate) !== null && _c !== void 0 ? _c : [];
        this.columnsForFilters = config.columnsForFilters || undefined;
        this.columnsForUpdate = config.columnsForUpdate || undefined;
    }
    async create(newEntityRow, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const insertRow = (0, knex_utils_1.pickWithoutUndefined)(newEntityRow, this.columnsForCreate);
        const insertedRows = await queryBuilder(this.tableName)
            .insert(insertRow)
            .returning(this.columnsToFetch);
        if (!(0, dbSupportHelper_1.doesSupportReturning)(this.knex)) {
            const insertedRow = await this.getById(insertedRows[0], undefined, transactionProvider);
            return insertedRow;
        }
        return insertedRows[0];
    }
    async createBulk(newEntityRows, transactionProvider, chunkSize = 1000) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const insertRows = newEntityRows.map((newEntityRow) => (0, knex_utils_1.pickWithoutUndefined)(newEntityRow, this.columnsForCreate));
        const chunks = (0, knex_utils_1.chunk)(insertRows, chunkSize);
        const insertedRows = [];
        for (const rows of chunks) {
            insertedRows.push(...(await queryBuilder(this.tableName).insert(rows).returning(this.columnsToFetch)));
        }
        return insertedRows;
    }
    async createBulkNoReturning(newEntityRows, transactionProvider, chunkSize = 1000) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const insertRows = newEntityRows.map((newEntityRow) => (0, knex_utils_1.pickWithoutUndefined)(newEntityRow, this.columnsForCreate));
        const chunks = (0, knex_utils_1.chunk)(insertRows, chunkSize);
        for (const rows of chunks) {
            await queryBuilder(this.tableName).insert(rows);
        }
    }
    async updateById(id, updatedFields, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const updatedColumns = this.columnsForUpdate
            ? (0, knex_utils_1.pickWithoutUndefined)(updatedFields, this.columnsForUpdate)
            : (0, knex_utils_1.copyWithoutUndefined)(updatedFields);
        const updatedUserRows = await queryBuilder(this.tableName)
            .where({ [this.idColumn]: id })
            .update(updatedColumns)
            .returning(this.columnsToFetch);
        return updatedUserRows[0];
    }
    async updateSingleByCriteria(filterCriteria, updatedFields, transactionProvider) {
        const result = await this.updateByCriteria(filterCriteria, updatedFields, transactionProvider);
        let returningResult;
        if ((0, dbSupportHelper_1.doesSupportReturning)(this.knex)) {
            returningResult = result;
        }
        else {
            returningResult = await this.getByCriteria(filterCriteria, undefined, this.columnsToFetchDetails, transactionProvider);
        }
        if (returningResult.length > 1) {
            throw new NonUniqueResultError_1.NonUniqueResultError('Query updated more than one row', filterCriteria);
        }
        if (returningResult.length === 0) {
            throw new NoEntityExistsError_1.NoEntityExistsError('Query updated no rows', filterCriteria);
        }
        return result[0];
    }
    async updateByCriteria(filterCriteria, updatedFields, transactionProvider, sorting) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const updatedColumns = this.columnsForUpdate
            ? (0, knex_utils_1.pickWithoutUndefined)(updatedFields, this.columnsForUpdate)
            : (0, knex_utils_1.copyWithoutUndefined)(updatedFields);
        const filters = this.columnsForFilters
            ? (0, knex_utils_1.pickWithoutUndefined)(filterCriteria, this.columnsForFilters)
            : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        const qb = queryBuilder(this.tableName)
            .where(filters)
            .update(updatedColumns)
            .returning(this.columnsToFetch);
        const sortParam = sorting !== null && sorting !== void 0 ? sorting : this.defaultOrderBy;
        if ((0, dbSupportHelper_1.doesSupportUpdateOrderBy)(this.knex) && sortParam) {
            qb.orderBy(sortParam);
        }
        const result = await qb;
        return result;
    }
    async getById(id, columnsToFetch, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        const result = await queryBuilder(this.tableName)
            .where({ [this.idColumn]: id })
            .select(columnsToFetch !== null && columnsToFetch !== void 0 ? columnsToFetch : this.columnsToFetchDetails);
        return result === null || result === void 0 ? void 0 : result[0];
    }
    async getByCriteria(filterCriteria, sorting, columnsToFetch, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        let filters;
        if (filterCriteria) {
            filters = this.columnsForFilters
                ? (0, knex_utils_1.pickWithoutUndefined)(filterCriteria, this.columnsForFilters)
                : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        }
        else {
            filters = {};
        }
        const qb = queryBuilder(this.tableName)
            .select(columnsToFetch !== null && columnsToFetch !== void 0 ? columnsToFetch : this.columnsToFetchList)
            .where(filters);
        const sortParam = sorting !== null && sorting !== void 0 ? sorting : this.defaultOrderBy;
        if (sortParam) {
            qb.orderBy(sortParam);
        }
        const result = await qb;
        return result;
    }
    async getSingleByCriteria(filterCriteria, columnsToFetch) {
        const result = await this.getByCriteria(filterCriteria, null, columnsToFetch !== null && columnsToFetch !== void 0 ? columnsToFetch : this.columnsToFetchDetails);
        if (result.length > 1) {
            throw new NonUniqueResultError_1.NonUniqueResultError('Query resulted more than in a single result', filterCriteria);
        }
        return result[0];
    }
    async deleteById(id, transactionProvider) {
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        await queryBuilder(this.tableName)
            .where({
            [this.idColumn]: id,
        })
            .del();
    }
    async deleteByCriteria(filterCriteria, transactionProvider) {
        const filters = this.columnsForFilters
            ? (0, knex_utils_1.pickWithoutUndefined)(filterCriteria, this.columnsForFilters)
            : (0, knex_utils_1.copyWithoutUndefined)(filterCriteria);
        const queryBuilder = await this.getKnexOrTransaction(transactionProvider);
        await queryBuilder(this.tableName).where(filters).del();
    }
    createTransactionProvider() {
        return this.knex.transactionProvider();
    }
    async commitTransaction(transactionProvider) {
        const trx = await transactionProvider();
        await trx.commit();
    }
    async rollbackTransaction(transactionProvider) {
        const trx = await transactionProvider();
        await trx.rollback();
    }
    async getKnexOrTransaction(transactionProvider) {
        return transactionProvider ? await transactionProvider() : this.knex;
    }
}
exports.AbstractRepository = AbstractRepository;
